/* ============================================================
 * Author: Aditya Nagrath
 * Date  : 2008-01-24
 * Copyright 2008 Palm, Inc. All rights reserved.
 * ============================================================ */

#include "DocCommander.h"
#include <sys/types.h>
#include <sys/shm.h>
#include <stdio.h>

void DrawPixelRGB888(int x, int y, int width,unsigned char R, unsigned char G, unsigned char B, void *buf)
{
  unsigned char *bufp = (unsigned char *)(buf) + y*(width*3) + x * 3;
  bufp[2] = R;
  bufp[1] = G;
  bufp[0] = B;    
}



DocCommander::DocCommander(char *filename, int id):
  mRenderer(filename),
  mNeedsPassword(0),
  mID(id),
  mOutline(NULL),
  mInitError(false)
{
  printf("Opening... %s with id %d\n", filename, id);
  mNeedsPassword = mRenderer.IsPasswordProtected();
  for(int i=0; i< MAX_SHARED_BUF; i++)
    {
      mRegisteredBuffers[i].shmid = -1;
      mRegisteredBuffers[i].buffer = NULL;
    }
  if(mRenderer.HasInitError())
    {
      mInitError = true;
    }
}

DocCommander::~DocCommander()
{

}

int DocCommander::RenderToBuffer(int x, int y, int width, int height, int shmid)
{
  int i=0;
  for(i=0;i<MAX_SHARED_BUF;i++)
    {   
      if(shmid == mRegisteredBuffers[i].shmid)
        break; //already registered
      if(mRegisteredBuffers[i].shmid == -1)
        break; //end of registered buffers... put it here!
    }
  if(shmid != mRegisteredBuffers[i].shmid)
  {
      fprintf(stderr, "No Registered Buffer with SHMID %d!\n",shmid);
      return -1;
  }

  return mRenderer.RenderCurrentPage(x,y,width,height, mRegisteredBuffers[i].buffer);  
}

int DocCommander::Initialize()
{
  int returnvalue;
  returnvalue = mRenderer.Initialize();
  if(returnvalue ==0)
    {
      mRenderer.SetRenderingMechanism(&DrawPixelRGB888);
      mRenderer.JumpToPage(1);
      mOutline = mRenderer.GetOutline();
    }
  return returnvalue;
}

void DocCommander::SetPassword(char *password)
{
  if(mRenderer.SetPassword(password)==0)
    mNeedsPassword = false;
  else
    mNeedsPassword = true;
}

bool DocCommander::NeedsPassword()
{
  return mNeedsPassword;
}

void DocCommander::RegisterBuffer(int shmid, size_t size)
{
  int i=0;
  for(i=0;i<MAX_SHARED_BUF;i++)
    {   
      if(shmid == mRegisteredBuffers[i].shmid)
        return; //already registered
      if(mRegisteredBuffers[i].shmid == -1)
        break; //end of registered buffers... put it here!
    }

  if(i==MAX_SHARED_BUF)//there is no room
    return; //return error code when comm back to java is figured out
    
  mRegisteredBuffers[i].shmid = shmid;
  mRegisteredBuffers[i].size = size;
  mRegisteredBuffers[i].buffer = shmat(shmid, NULL, 0);
  printf("Registered Buffer %d at %d\n", shmid, i);
}

int DocCommander::NextPage()
{
  return mRenderer.NextPage();
}

int DocCommander::PreviousPage()
{
  return mRenderer.PreviousPage();
}

int DocCommander::JumpToPage(int page)
{
  return mRenderer.JumpToPage(page);
}

void DocCommander::SetZoom(double inZoom)
{
  mRenderer.SetZoom(inZoom);
}

//if there is a return value... then let the server handle the return
//if there is more to send back we'll handle it here..
double DocCommander::GetZoom()
{
  return mRenderer.GetZoom();
}

void DocCommander::SetRotation(int inRotation)
{
  mRenderer.SetRotation(inRotation);
}

int DocCommander::GetRotation()
{
  return mRenderer.GetRotation();
}

int DocCommander::GetCurrentPageNumber()
{
  return mRenderer.GetCurrentPage()->GetPageNumber();
}

int DocCommander::GetPageCount()
{
  return mRenderer.GetPageCount();
}

void DocCommander::GetPageDimensions(int &width, int &height)
{
  width =  (int)mRenderer.GetCurrentPage()->GetWidth();
  height = (int)mRenderer.GetCurrentPage()->GetHeight();
}

int DocCommander::HasLinkAtLocation(int x, int y, char *buffer, bool getlink)
{
  return mRenderer.HasLinkAtLocation(x, y, buffer, getlink);
}

void DocCommander::OutlineRoot()
{
  mOutline->GotoRoot();
}

char *DocCommander::GetOutlineNodeText()
{
  return mOutline->GetCurrentNodeText();
}

void DocCommander::OutlineGotoNextSibling()
{
  mOutline->GotoNextSibling();
}

void DocCommander::OutlineGotoChild()
{
  mOutline->GotoChild();
}

bool DocCommander::OutlineHasNextSibling()
{
  return mOutline->HasNextSibling();
}

bool DocCommander::OutlineHasChild()
{
  return mOutline->HasChild();
}

int DocCommander::GetOutlineLinkPageNumber()
{
  return mOutline->GetCurrentNodeLinkPageNumber();
}

void DocCommander::CaseInsensitiveTextSearch(char *needle,int startpage, int endpage,
					      void (*RecordResult)(int page, int x0, int y0, int x1, int y1, int contextlen, char *context, void *), void *userdata)
{
  mRenderer.CaseInsensitiveTextSearch(needle, strlen(needle), startpage, endpage, RecordResult, userdata);
}

void DocCommander::DumpCurrentPageText()
{
  mRenderer.GetCurrentPage()->DebugPageText();
}

void DocCommander::TranslateScreenPoint(int &x, int &y)
{
  fz_point mousepoint = mRenderer.TranslateScreenPoint(x, y);
  x = (int) mousepoint.x;
  y = (int) mousepoint.y;

}

void DocCommander::TranslateScreenDelta(int &x, int &y)
{
  fz_point mousepoint = mRenderer.TranslateScreenDelta(x, y);
  x = (int) mousepoint.x;
  y = (int) mousepoint.y;

}
