
#ifndef PDFPAGE_H
#define PDFPAGE_H

#include "PDF.h"
#include "mustd.h"

class PDF;

/// PDFPage class.  Represents one page of the PDF
class PDFPage
{
public:
    /// PDFPage() Pass in the PDF file this page belongs to
    /// and the pagenumber you wish to construct.
    PDFPage(PDF *parent,int pagenumber);
    ~PDFPage();

    /// GetPageNumber() returns the page number associated with this page
    int GetPageNumber();
    /// Loaded() -- true indicates loaded without error, false indicates error
    bool Loaded();
    /// GetWidth() and GetHeight() The "native" width, height of the page
    float GetWidth();
    float GetHeight();

    /// GetRotation() -- returns the "native" rotation of the page
    int GetRotation();

    /// Print all the text of the PDFPage to console
    void DebugPageText();
    /// CaseInsensitiveSearchPageText()
    /// needle is the text we are searching for
    /// needlelength is the strlen of needle
    /// badchar is the "bad character match table" required for boyer-moore
    /// goodsuff is the "good suffix" table required for boyer-moore
    /// RecordResult is a callback function called when a match is found, 
    /// See PDF.h for more details.
    /// userdata gets passed back to recordresult for convenience.
    void CaseInsensitiveSearchPageText(char *needle, int needlelength, int *badchar, int* goodsuff,
				       void (*RecordResult)(int, int, int, int, int, int, char *, void *), void *userdata);

private:
    void DebugTextLine(pdf_textline *line);

    pdf_page *mPage;
    PDF *mParent;
    int mPageNumber;
    friend class PDF; 
    friend class PDFRenderer;
};
#endif
