#include "PDFRenderer.h"

using namespace std;

 

PDFRenderer::PDFRenderer(char *filename):
    PDF(filename),
    mGC(NULL),
    mDrawZoom(1),
    mDrawRotation(0),
    DrawPixel(NULL)
{
    fz_error *error;
    // XXX Figure out what hte 1024x512 is about
    error = fz_newrenderer(&mGC, pdf_devicergb, 0, 512 * 512);
    if (error)
        die(error);
}

PDFRenderer::~PDFRenderer()
{
    fz_droprenderer(mGC);
}

void PDFRenderer::SetZoom(double inZoom)
{
    mDrawZoom = inZoom;
}

double PDFRenderer::GetZoom()
{
    return mDrawZoom;
}

void PDFRenderer::SetRotation(int inRotation)
{
    mDrawRotation = inRotation;
}

int PDFRenderer::GetRotation()
{
    return mDrawRotation;
}

void PDFRenderer::SetRenderingMechanism(void (*inFunc)(int, int, int,unsigned char, unsigned char, unsigned char, void *))
{ 
    DrawPixel = inFunc;
}

fz_point PDFRenderer::TranslateScreenPoint(int x, int y)
{
    fz_point actual_point;
    //translate screen point to point about origin (previously passed in center)
    actual_point.x = mCurrentTopLeft.x + x;
    actual_point.y = mCurrentTopLeft.y + y;
    
   //undo zoom/rotation
    fz_matrix inverted = fz_identity();
    inverted = fz_concat(inverted, fz_scale(mDrawZoom, -mDrawZoom));
    inverted = fz_concat(inverted, fz_rotate(mDrawRotation + mCurrentPage->GetRotation()));
    inverted = fz_invertmatrix(inverted);
    return fz_transformpoint(inverted, actual_point);
}

fz_point PDFRenderer::TranslateScreenDelta(int x, int y)
{
    fz_point actual_point;
    //In this function we were handed the motion vector.. just fix it!
    actual_point.x = x;
    actual_point.y = y;
    
   //undo zoom/rotation
    fz_matrix inverted = fz_identity();

    inverted = fz_concat(inverted, fz_scale(mDrawZoom, -mDrawZoom));
    inverted = fz_concat(inverted, fz_rotate(mDrawRotation + mCurrentPage->GetRotation()));
    inverted = fz_invertmatrix(inverted);
    return fz_transformpoint(inverted, actual_point);
}

int PDFRenderer::RenderCurrentPage(int x, int y, int width, int height, void *mBuf)
{
    fz_error *error;
    fz_pixmap *pix;

    if(mCurrentPage == NULL)
        return -1;
   
    mCtm = fz_identity();

    int midx=0;
    int midy=0;

    if(mCurrentPage->Loaded())
    {
        midx = (int)(x+mCurrentPage->mPage->mediabox.x0);
        midy = (int)(y+mCurrentPage->mPage->mediabox.y0);
    }
    //Translate user center to 0,0
    mCtm = fz_concat(mCtm, fz_translate(-midx, -midy));
    //zoom and rotate and stuff!
    mCtm = fz_concat(mCtm, fz_scale(mDrawZoom, -mDrawZoom));
    mCtm = fz_concat(mCtm, fz_rotate(mDrawRotation + mCurrentPage->GetRotation()));
    if(mCurrentPage->Loaded())
        mBbox = fz_roundrect(fz_transformaabb(mCtm, mCurrentPage->mPage->mediabox));    
 
     
    // So they passed in the center, lets find the real top left!
    mCurrentTopLeft.x = (0 - (width/2));
    mCurrentTopLeft.y = (0 - (height/2));

    error = fz_newpixmap(&pix, (int)mCurrentTopLeft.x, (int)mCurrentTopLeft.y, width, height, 4);
    if (error)
      return -1;


    memset(pix->samples, 0xff, pix->h * pix->w * pix->n);
    if(mCurrentPage->Loaded())
    {
        error = fz_rendertreeover(mGC, pix, mCurrentPage->mPage->tree, mCtm);
        if (error)
            return -1;
    }

    if(DrawPixel != NULL)
    {
        for (y = 0; y < pix->h; y++)
        {
	  unsigned char *src = pix->samples + y * pix->w * 4;
	  for (x = 0; x < pix->w; x++)
	  {		
	    DrawPixel(x,y, width, src[x*4+1], src[x*4+2], src[x*4+3], mBuf);
	  }
	  memset(src, 0xff, pix->w * 4);
        }
    }
    fz_droppixmap(pix);
    return 0;
}
